@@ -31,6 +31,11 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <sys/wait.h>
+#if defined(USE_POLL)
+#include <poll.h>
+#else
+#include <sys/select.h>
+#endif
 #include "net_io.h"
 #include "proc_mutex.h"
 #include "debug.h"
@@ -464,43 +469,26 @@ void ci_named_pipe_close(int pipe_fd)
 
 int wait_for_commands(int ctl_fd, char *command_buffer, int secs)
 {
-     fd_set fds;
-     struct timeval tv;
-     int ret = 0;
+    int ret = 0;
+    if ((ret = ci_wait_for_data(ctl_fd, secs, wait_for_read)) > 0) {
+        ret = ci_read_nonblock(ctl_fd, command_buffer, COMMANDS_BUFFER_SIZE - 1);
+        if (ret > 0) {
+            command_buffer[ret] = '\0';
+            return ret;
+        }
+        if (ret == 0 ) {
+            /*read return 0, This is an eof, we must return -1 to force socket reopening!*/
+            return -1;
+        }
+    }
+    if (ret < 0) {
+        ci_debug_printf(1,
+                        "Unexpected error waiting for or reading  events in control socket!\n");
+        /*returning -1 we are causing reopening control socket! */
+        return -1;
+    }
 
-     if (secs >= 0) {
-          tv.tv_sec = secs;
-          tv.tv_usec = 0;
-     }
-     FD_ZERO(&fds);
-     FD_SET(ctl_fd, &fds);
-     errno = 0;
-     if ((ret =
-          select(ctl_fd + 1, &fds, NULL, NULL,
-                 (secs >= 0 ? &tv : NULL))) > 0) {
-          if (FD_ISSET(ctl_fd, &fds)) {
-               ret =
-                   ci_read_nonblock(ctl_fd, command_buffer,
-                                    COMMANDS_BUFFER_SIZE - 1);
-               if (ret > 0) {
-                   command_buffer[ret] = '\0';
-                   return ret;
-               }
-
-               if (ret == 0 ) {
-                /*read return 0, This is an eof, we must return -1 to force socket reopening!*/
-                   return -1;
-               }
-          }
-     }
-     if (ret < 0 && errno != EINTR) {
-          ci_debug_printf(1,
-                          "Unexpected error waiting for or reading  events in control socket!\n");
-          /*returning -1 we are causing reopening control socket! */
-          return -1;
-     }
-
-     return 0; /*expired */
+    return 0; /*expired */
 }
 
 void handle_monitor_process_commands(char *cmd_line)
@@ -720,13 +708,20 @@ void listener_thread(int *fd)
           ci_debug_printf(7, "Child %d getting requests now ...\n", pid);
           do {                  //Getting requests while we have free servers.....
 #ifndef SINGLE_ACCEPT
-               fd_set fds;
-               int ret;
                do {
+                    int ret;
+                    errno = 0;
+#if defined(USE_POLL)
+                    struct pollfd pfds[1];
+                    pfds[0].fd = sockfd;
+                    pfds[0].events = POLLIN;
+                    ret = poll(pfds, 1, -1);
+#else
+                    fd_set fds;
                     FD_ZERO(&fds);
                     FD_SET(sockfd, &fds);
-                    errno = 0;
                     ret = select(sockfd + 1, &fds, NULL, NULL, NULL);
+#endif
                     if (ret < 0) {
                          if (errno != EINTR) {
                               ci_debug_printf(1,
